/***********************************************************************
 *    DecoEmojiListDao
 *
 * [File]
 *     DecoEmojiListDao.java
 *
 *     Copyright (C) 2011-2012 OMRON SOFTWARE Co., Ltd.  All Rights Reserved.
 *
 * [Description]
 *     DecoEmojiList Data Access Object Class.
 ************************************************************************/
package jp.co.omronsoft.android.decoemojilist_docomo;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

import jp.co.omronsoft.android.decoemojilist_docomo.constants.Const;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * 一覧表示制御アプリ 入力履歴テーブル操作用DAOクラス.
 */
public class DecoEmojiListDao {

    /** For Debug Class Name. */
    private static final String CLASS_NAME = "DecoEmojiListDao#";

    /** DB操作用クラスを表すフィールド. */
    private SQLiteDatabase mDatabase = null;

    /** ローカルコンテキスト(DecoEmojiList)を表すフィールド. */
    private Context mLocalContext = null;

    /** 入力履歴のページ数を表すフィールド. */
    private int mDecoHistoryPageSize = 0;

    /** 入力履歴テーブルのレコードを表すフィールド. */
    List<String> mDecoHistories;

    /** 入力履歴テーブルのレコードが存在するかを表すフィールド. */
    private boolean mIsExistDecoHistory = false;

    /** 1画面に表示するデコ絵文字数を表すフィールド. */
    private int mDecoEmojiCount = 0;

    /** DB操作用クラスを宣言する. */
    private class DatabaseHelper extends SQLiteOpenHelper {

        /** コンストラクタ. */
        DatabaseHelper(Context context) {
            super(context, Const.DB_NAME, null, Const.DB_VERSION);
        }

        /** @see android.database.sqlite.SQLiteOpenHelper#onCreate */
        @Override
        public void onCreate(SQLiteDatabase db) {
            try {
                db.execSQL(Const.SQL_CREATE_TABLE_DECO);
                db.execSQL(Const.SQL_CREATE_TABLE_CARRIER);
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }

        /** @see android.database.sqlite.SQLiteOpenHelper#onUpgrade */
        @Override
        public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
            try {
                db.execSQL(Const.SQL_DROP_TABLE_DECO);
                db.execSQL(Const.SQL_DROP_TABLE_CARRIER);
            } catch (SQLException e) {
                e.printStackTrace();
            }
            onCreate(db);
        }
    }

    /** コンストラクタ. */
    DecoEmojiListDao(Context context) {
        mLocalContext = context;
        mDecoHistoryPageSize = 0;
        openHistories();
    }

    /** DBの接続を行う. */
    private void openHistories() {
        closeHistories();
        DatabaseHelper dbHelper = new DatabaseHelper(mLocalContext);
        mDatabase = dbHelper.getWritableDatabase();
    }

    /** DBの切断を行う. */
    void closeHistories() {
        if (mDatabase != null) {
            mDatabase.close();
            mDatabase = null;
        }
    }

    /**
     * 入力履歴が存在するかを取得する.
     *
     * @return 存在フラグ(true:存在する/false:存在しない)
     */
    boolean isExistHistories() {
        return mIsExistDecoHistory;
    }

    /**
     * 初期処理を行う.
     *
     * @param pageSize
     *            1ページに表示するデコ絵文字数
     */
    void init(int decoEmojiCount) {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "init() start. decoEmojiCount[" + decoEmojiCount
                    + "]");
        }

        mDecoEmojiCount = decoEmojiCount;
        if (mDecoEmojiCount != 0) {
            int rowCount;
            SQLiteDatabase con = mDatabase;

            Cursor cursor = con.query(Const.TABLE_NAME_DECO, new String[] {
                    Const.COLUMN_HISTORY_ID, Const.COLUMN_URI }, null, null, null, null,
                    Const.ORDER_BY_HISTORY_ID_ASC);
            rowCount = cursor.getCount();
            
            int cnt  = 0;
            cursor.moveToFirst();
            for (int i = 0; i < rowCount; i++) {
            	            String uri = cursor.getString(Const.COLUMN_INDEX_URI);
            	            File file = new File(uri);
            	            if (file.exists()) {
            	            	cnt++;
            	            } 
            	            cursor.moveToNext();
            	        }
            cursor.close();

            if (cnt > 0) {
                // 切り上げで計算する
                mDecoHistoryPageSize = rowCount / mDecoEmojiCount;
                if ((rowCount % mDecoEmojiCount) > 0) {
                    mDecoHistoryPageSize += 1;
                }
                mIsExistDecoHistory = true;

                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME + "init() mIsExistHistory[" + mIsExistDecoHistory
                            + "] mHistoryPageSize[" + mDecoHistoryPageSize + "]");
                }
            }
        }

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "init() end.");
        }
    }

    /**
     * 入力履歴のページ数を取得する.
     *
     * @return 入力履歴のページ数
     */
    int getHistoryPageSize() {
        return mDecoHistoryPageSize;
    }

    /**
     * 入力履歴テーブルのレコードを取得する.
     *
     * @param pageNum
     *            取得するページ数
     * @return 入力履歴テーブルのレコード
     */
    List<String> loadHistories(int pageNum) {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "loadHistories() start. pageNum[" + pageNum + "]");
        }

        int rowCount = 0;

        // 取得する入力履歴の範囲を算出する(limitからoffset分取得する)
        int limit = pageNum * mDecoEmojiCount;
        int offset = mDecoEmojiCount;

        // limitがマイナスとなる検索はエラーとなる為
        if (limit < 0) {
            if (Const.DEBUG) {
                Log.d(Const.TAG, CLASS_NAME + "loadHistories() end. 'limit' Is Minus.");
            }
            return null;
        }

        SQLiteDatabase con = mDatabase;

        mDecoHistories = new ArrayList<String>();
        Cursor cursor = con.query(Const.TABLE_NAME_DECO, new String[] { Const.COLUMN_HISTORY_ID,
                Const.COLUMN_URI }, null, null, null, null, Const.ORDER_BY_HISTORY_ID_DESC,
                limit + "," + offset);
        rowCount = cursor.getCount();

        if (rowCount == 0) {
            cursor.close();

            if (Const.DEBUG) {
                Log.d(Const.TAG, CLASS_NAME + "loadHistories() end. No Histories.");
            }
            return null;
        }

        cursor.moveToFirst();
        for (int i = 0; i < rowCount; i++) {
            String uri = cursor.getString(Const.COLUMN_INDEX_URI);
            File file = new File(uri);
            if (file.exists()) {
                mDecoHistories.add(cursor.getString(Const.COLUMN_INDEX_URI));
            } 
            cursor.moveToNext();
        }
        cursor.close();

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "loadHistories() end.");
        }

        return mDecoHistories;
    }

    /**
     * 入力履歴テーブルを更新する.
     *
     * @param uri
     *            デコ絵文字のURI
     */
    void updateHistory(String uri) {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "updateHistory() start. uri[" + uri + "]");
        }

        // 2度押し対策
        if (mDatabase == null) {
            Log.e(Const.TAG, "mDatabase is null.");
            return;
        }

        int i = 0;
        SQLiteDatabase con = mDatabase;

        Cursor cursor = con.query(Const.TABLE_NAME_DECO, new String[] { Const.COLUMN_HISTORY_ID,
                Const.COLUMN_URI }, null, null, null, null, Const.ORDER_BY_HISTORY_ID_ASC);

        int historyCount = cursor.getCount();
        if (Const.DB_MAX_HISTORY_RECORD < historyCount) {
            historyCount = Const.DB_MAX_HISTORY_RECORD;
        }

        int lastRowId = 0;

        if (cursor.moveToLast()) {

            // 最終レコードのhistory_idを取得する
            lastRowId = cursor.getInt(Const.COLUMN_INDEX_HISTORY_ID);

            // IDが上限となった時、全レコードを1から採番する
            if (lastRowId == Const.MAX_HISTORY_ID) {

                cursor.moveToFirst();
                ContentValues args = new ContentValues();
                int rowId = 0;

                con.beginTransaction();
                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME
                            + "updateHistory() MaxHistoryId Update Transaction Begin.");
                }

                for (i = 0; i < historyCount; i++) {
                    rowId = cursor.getInt(Const.COLUMN_INDEX_HISTORY_ID);
                    args.put(Const.COLUMN_HISTORY_ID, i);
                    con.update(Const.TABLE_NAME_DECO, args, Const.COLUMN_HISTORY_ID + "=" + rowId,
                            null);
                    cursor.moveToNext();
                }

                con.setTransactionSuccessful();
                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME
                            + "updateHistory() MaxHistoryId Update Transaction Successful.");
                }

                con.endTransaction();
                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME
                            + "#updateHistory() MaxHistoryId Update Transaction End.");
                }

                cursor.close();

                cursor = con.query(Const.TABLE_NAME_DECO, new String[] { Const.COLUMN_HISTORY_ID,
                        Const.COLUMN_URI }, null, null, null, null,
                        Const.ORDER_BY_HISTORY_ID_ASC);
                lastRowId = i;
            }
            lastRowId++;
        } else {
            lastRowId = 1;
        }

        // 既に登録されているURIであった時、該当レコードを最終レコードとして更新する
        if (0 < historyCount) {

            int dbHistoryId = 0;
            cursor.moveToFirst();

            // 取得できた件数分、検索を行う
            for (i = 0; i < historyCount; i++) {
                String dbUri = cursor.getString(Const.COLUMN_INDEX_URI);
                if (dbUri.equalsIgnoreCase(uri)) {
                    dbHistoryId = cursor.getInt(Const.COLUMN_INDEX_HISTORY_ID);
                    cursor.close();
                    // 1件目で同一のURIがヒットした時は、更新する必要なし
                    if ((i + 1) != historyCount) {

                        con.beginTransaction();
                        if (Const.DEBUG) {
                            Log.d(Const.TAG, CLASS_NAME
                                    + "updateHistory() IsExistUri Update Transaction Begin.");
                        }

                        ContentValues args = new ContentValues();
                        args.put(Const.COLUMN_HISTORY_ID, lastRowId);
                        con.update(Const.TABLE_NAME_DECO, args, Const.COLUMN_HISTORY_ID + "="
                                + dbHistoryId, null);

                        con.setTransactionSuccessful();
                        if (Const.DEBUG) {
                            Log.d(Const.TAG,
                                    CLASS_NAME
                                            + "updateHistory() IsExistUri Update Transaction Successful.");
                        }

                        con.endTransaction();
                        if (Const.DEBUG) {
                            Log.d(Const.TAG, CLASS_NAME
                                    + "updateHistory() IsExistUri Update Transaction End.");
                        }
                    }

                    if (Const.DEBUG) {
                        Log.d(Const.TAG, CLASS_NAME + "updateHistory() end. IsExistUri.");
                    }
                    return;
                }
                cursor.moveToNext();
            }

            // 登録数が上限となった時、最も古いレコードを削除する
            if (i >= Const.DB_MAX_HISTORY_RECORD) {

                con.beginTransaction();
                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME
                            + "updateHistory() MaxHistoryRecord Delete Transaction Begin.");
                }

                cursor.moveToFirst();
                dbHistoryId = cursor.getInt(Const.COLUMN_INDEX_HISTORY_ID);
                con.delete(Const.TABLE_NAME_DECO, Const.COLUMN_HISTORY_ID + "=" + dbHistoryId, null);

                con.setTransactionSuccessful();
                if (Const.DEBUG) {
                    Log.d(Const.TAG,
                            CLASS_NAME
                                    + "updateHistory() MaxHistoryRecord Delete Transaction Successful.");
                }

                con.endTransaction();
                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME
                            + "updateHistory() MaxHistoryRecord Delete Transaction End.");
                }
            }
        }
        cursor.close();

        con.beginTransaction();
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME
                    + "updateHistory() NewHistoryRecord Insert Transaction Begin.");
        }

        // 入力履歴を最終レコードに登録する
        ContentValues cv = new ContentValues();
        cv.put(Const.COLUMN_HISTORY_ID, lastRowId);
        cv.put(Const.COLUMN_URI, uri);
        con.insert(Const.TABLE_NAME_DECO, null, cv);

        con.setTransactionSuccessful();
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME
                    + "updateHistory() NewHistoryRecord Insert Transaction Successful.");
        }

        con.endTransaction();
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME
                    + "updateHistory() NewHistoryRecord Insert Transaction End.");
        }

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "updateHistory() end.");
        }
    }
    
    /**
     * キャリア絵文字入力履歴が存在するかを取得する.
     *
     * @return 存在フラグ(true:存在する/false:存在しない)
     */
    boolean isExistHistoriesCarrier() {
        
        boolean isExist = false;
        
        SQLiteDatabase con = mDatabase;

        Cursor cursor = con.query(Const.TABLE_NAME_CARRIER, new String[] {
                Const.COLUMN_HISTORY_ID, Const.COLUMN_CODE }, null, null, null, null, null);
        int rowCount = cursor.getCount();
        cursor.close();

        if (rowCount > 0) {
            isExist = true;
        }
        
        return isExist;
    }
    
    /**
     * キャリア絵文字入力履歴テーブルのレコードを取得する.
     *
     * @return 入力履歴テーブルのレコード
     */
    List<String> loadHistoriesCarrier() {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "loadHistories() start. ");
        }

        int rowCount = 0;

        SQLiteDatabase con = mDatabase;

        mDecoHistories = new ArrayList<String>();
    
        Cursor cursor = con.query(Const.TABLE_NAME_CARRIER, new String[] { Const.COLUMN_HISTORY_ID,
                Const.COLUMN_CODE }, null, null, null, null, Const.ORDER_BY_HISTORY_ID_DESC);
        
        rowCount = cursor.getCount();

        if (rowCount == 0) {
            cursor.close();

            if (Const.DEBUG) {
                Log.d(Const.TAG, CLASS_NAME + "loadHistories() end. No Histories.");
            }
            return null;
        }

        cursor.moveToFirst();
        for (int i = 0; i < rowCount; i++) {
            mDecoHistories.add(i, cursor.getString(Const.COLUMN_INDEX_CODE));
            cursor.moveToNext();
        }
        cursor.close();

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "loadHistories() end.");
        }

        return mDecoHistories;
    }
    
    /**
     * キャリア絵文字入力履歴テーブルを更新する.
     *
     * @param code
     *            キャリア絵文字の文字コード
     */
    void updateHistoryCarrier(String code) {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "updateHistory() start. code[" + code + "]");
        }

        // 2度押し対策
        if (mDatabase == null) {
            Log.e(Const.TAG, "mDatabase is null.");
            return;
        }

        int i = 0;
        SQLiteDatabase con = mDatabase;

        Cursor cursor = con.query(Const.TABLE_NAME_CARRIER, new String[] { Const.COLUMN_HISTORY_ID,
                Const.COLUMN_CODE }, null, null, null, null, Const.ORDER_BY_HISTORY_ID_ASC);

        int historyCount = cursor.getCount();
        if (Const.DB_MAX_HISTORY_RECORD < historyCount) {
            historyCount = Const.DB_MAX_HISTORY_RECORD;
        }

        int lastRowId = 0;

        if (cursor.moveToLast()) {

            // 最終レコードのhistory_idを取得する
            lastRowId = cursor.getInt(Const.COLUMN_INDEX_HISTORY_ID);

            // IDが上限となった時、全レコードを1から採番する
            if (lastRowId == Const.MAX_HISTORY_ID) {

                cursor.moveToFirst();
                ContentValues args = new ContentValues();
                int rowId = 0;

                con.beginTransaction();
                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME
                            + "updateHistory() MaxHistoryId Update Transaction Begin.");
                }

                for (i = 0; i < historyCount; i++) {
                    rowId = cursor.getInt(Const.COLUMN_INDEX_HISTORY_ID);
                    args.put(Const.COLUMN_HISTORY_ID, i);
                    con.update(Const.TABLE_NAME_CARRIER, args, Const.COLUMN_HISTORY_ID + "=" + rowId,
                            null);
                    cursor.moveToNext();
                }

                con.setTransactionSuccessful();
                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME
                            + "updateHistory() MaxHistoryId Update Transaction Successful.");
                }

                con.endTransaction();
                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME
                            + "#updateHistory() MaxHistoryId Update Transaction End.");
                }

                cursor.close();

                cursor = con.query(Const.TABLE_NAME_CARRIER, new String[] { Const.COLUMN_HISTORY_ID,
                        Const.COLUMN_CODE }, null, null, null, null,
                        Const.ORDER_BY_HISTORY_ID_ASC);
                lastRowId = i;
            }
            lastRowId++;
        } else {
            lastRowId = 1;
        }

        // 既に登録されているURIであった時、該当レコードを最終レコードとして更新する
        if (0 < historyCount) {

            int dbHistoryId = 0;
            cursor.moveToFirst();

            // 取得できた件数分、検索を行う
            for (i = 0; i < historyCount; i++) {
                String dbUri = cursor.getString(Const.COLUMN_INDEX_CODE);
                if (dbUri.equalsIgnoreCase(code)) {
                    dbHistoryId = cursor.getInt(Const.COLUMN_INDEX_HISTORY_ID);
                    cursor.close();
                    // 1件目で同一のURIがヒットした時は、更新する必要なし
                    if ((i + 1) != historyCount) {

                        con.beginTransaction();
                        if (Const.DEBUG) {
                            Log.d(Const.TAG, CLASS_NAME
                                    + "updateHistory() IsExistUri Update Transaction Begin.");
                        }

                        ContentValues args = new ContentValues();
                        args.put(Const.COLUMN_HISTORY_ID, lastRowId);
                        con.update(Const.TABLE_NAME_CARRIER, args, Const.COLUMN_HISTORY_ID + "="
                                + dbHistoryId, null);

                        con.setTransactionSuccessful();
                        if (Const.DEBUG) {
                            Log.d(Const.TAG,
                                    CLASS_NAME
                                            + "updateHistory() IsExistUri Update Transaction Successful.");
                        }

                        con.endTransaction();
                        if (Const.DEBUG) {
                            Log.d(Const.TAG, CLASS_NAME
                                    + "updateHistory() IsExistUri Update Transaction End.");
                        }
                    }

                    if (Const.DEBUG) {
                        Log.d(Const.TAG, CLASS_NAME + "updateHistory() end. IsExistUri.");
                    }
                    return;
                }
                cursor.moveToNext();
            }

            // 登録数が上限となった時、最も古いレコードを削除する
            if (i >= Const.DB_MAX_HISTORY_RECORD) {

                con.beginTransaction();
                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME
                            + "updateHistory() MaxHistoryRecord Delete Transaction Begin.");
                }

                cursor.moveToFirst();
                dbHistoryId = cursor.getInt(Const.COLUMN_INDEX_HISTORY_ID);
                con.delete(Const.TABLE_NAME_CARRIER, Const.COLUMN_HISTORY_ID + "=" + dbHistoryId, null);

                con.setTransactionSuccessful();
                if (Const.DEBUG) {
                    Log.d(Const.TAG,
                            CLASS_NAME
                                    + "updateHistory() MaxHistoryRecord Delete Transaction Successful.");
                }

                con.endTransaction();
                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME
                            + "updateHistory() MaxHistoryRecord Delete Transaction End.");
                }
            }
        }
        cursor.close();

        con.beginTransaction();
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME
                    + "updateHistory() NewHistoryRecord Insert Transaction Begin.");
        }

        // 入力履歴を最終レコードに登録する
        ContentValues cv = new ContentValues();
        cv.put(Const.COLUMN_HISTORY_ID, lastRowId);
        cv.put(Const.COLUMN_CODE, code);
        con.insert(Const.TABLE_NAME_CARRIER, null, cv);

        con.setTransactionSuccessful();
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME
                    + "updateHistory() NewHistoryRecord Insert Transaction Successful.");
        }

        con.endTransaction();
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME
                    + "updateHistory() NewHistoryRecord Insert Transaction End.");
        }

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "updateHistory() end.");
        }
    }
}
