/***********************************************************************
 *    DecoEmojiListViewManager
 *
 * [File]
 *     DecoEmojiListViewManager.java
 *
 *     Copyright (C) 2011-2012 OMRON SOFTWARE Co., Ltd.  All Rights Reserved.
 *
 * [Description]
 *     DecoEmojiList View Manager Class.
 ************************************************************************/
package jp.co.omronsoft.android.decoemojilist_docomo;

import android.content.res.Resources;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.Spinner;
import android.widget.TextView;

import jp.co.omronsoft.android.decoemojilist_docomo.constants.Const;

/**
 * 一覧表示制御アプリ 描画クラス.
 */
public class DecoEmojiListViewManager {

    /** For Debug Class Name. */
    private static final String CLASS_NAME = "DecoEmojiListViewManager#";

    /** 1行に表示する最大絵文字数(列). */
    private int mDecoEmojiMaxCols;

    /** 1画面に表示する最大絵文字数(行). */
    private int mDecoEmojiMaxRows;

    /** ディスプレイの横幅を表すフィールド. */
    private int mDisplayWidth;

    /** デコ絵文字一覧が表示されるLinearの縦幅を表すフィールド. */
    private int mDecoEmojiLinearHeight;

    /** デコ絵文字一覧の最大描画数を表すフィールド. */
    private int mDecoEmojiMaxDrawableCount;

    /** デコ絵文字の横幅を表すフィールド. */
    private int mDecoEmojiWidth;

    /** 1行の右端にあるデコ絵文字(単体)の横幅を表すフィールド. */
    private int mDecoEmojiWidthEndOfLine;

    /** デコ絵文字の縦幅を表すフィールド. */
    private int mDecoEmojiHeight;

    /** 1画面の最終にあるデコ絵文字(行)の縦幅を表すフィールド. */
    private int mDecoEmojiHeightEndOfLine;

    /** 1行あたりのピクセル数を表すフィールド. */
    private int mDisplayOneLineHeight;

    /** ディスプレイの縦幅を表すフィールド. */
    private int mDisplayHeight;

    /** メインビューのグループを表すフィールド. */
    private ViewGroup mViewGroup;

    /** メインビューの土台のグループを表すフィールド. */
    private ViewGroup mListViewBase;

    /** プレビューテキストを表すフィールド. */
    private TextView mPreviewText;

    /** カテゴリ名称を表すフィールド. */
    private Spinner mCategoryName;

    /** デコ絵文字一覧を表示するViewを表すフィールド. */
    private ScrollView mBaseScroll;

    /** 確定ボタンを表すフィールド. */
    private ImageView mEnterButton;

    /** カテゴリ内ページ切り替え(前)ボタンを表すフィールド. */
    private ImageView mReadMoreButtonBack;

    /** カテゴリ内ページ切り替え(次)ボタンを表すフィールド. */
    private ImageView mReadMoreButtonNext;

    /** バックスペースボタンを表すフィールド. */
    private ImageView mBackSpaceButton;

    /** 縦／横画面(true:縦画面/false:横画面)を表すフィールド */
    private boolean mIsPort;

    /** キャリア絵文字、デコレーション絵文字の表示個数切替用 */
    private static final int INPUT_TYPE_SINGLE = 1;

    /** コンストラクタ. */
    DecoEmojiListViewManager() {
    }

    /**
     * DecoEmojiListViewの画面を整形する.
     *
     * @param res
     *            ActivityのgetResources()
     * @param inflater
     *            ActivityのgetLayoutInflater()
     * @param width
     *            ディスプレイの横幅
     * @param height
     *            ディスプレイの縦幅
     * @param currentImeHeight
     *            [0] 縦画面時の縦幅 [1] 横画面時の縦幅
     * @return 整形した土台(ViewGroup)
     */
    ViewGroup initView(Resources res, LayoutInflater inflater, int width, int height,
            int[] currentImeHeight, int mCurrentInputType) {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "initView() start. width[" + width + "] height["
                    + height + "]");
        }

        mDisplayWidth = width;
        mDisplayOneLineHeight = res.getDimensionPixelSize(R.dimen.display_one_line_height);

        // 回転の向きによって出力するデコ絵文字数を変更する
        mIsPort = (width < height);
        if (mIsPort) {
            mDisplayHeight = currentImeHeight[0];
            mDecoEmojiMaxCols = Const.DECO_EMOJI_MAX_COLS_HVGA;
            mDecoEmojiMaxRows = Const.DECO_EMOJI_MAX_ROWS_PORT;
            // 1個確定モードの場合はボタンエリアに2行プラス
            if (mCurrentInputType == INPUT_TYPE_SINGLE) {
                mDecoEmojiMaxRows = Const.DECO_EMOJI_MAX_ROWS_PORT + 2;
            }
        } else {
            mDisplayHeight = currentImeHeight[1];
            mDecoEmojiMaxCols = res.getInteger(R.integer.deco_emoji_max_cols);
            mDecoEmojiMaxRows = Const.DECO_EMOJI_MAX_ROWS_LAND;
            // 1個確定モードの場合はボタンエリアに2行プラス
            if (mCurrentInputType == INPUT_TYPE_SINGLE) {
                mDecoEmojiMaxRows = Const.DECO_EMOJI_MAX_ROWS_LAND + 2;
            }
        }

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "initView() mDisplayWidth[" + mDisplayWidth
                    + "] mDisplayHeight[" + mDisplayHeight + "] sDecoEmojiMaxCols["
                    + mDecoEmojiMaxCols + "] sDecoEmojiMaxRows[" + mDecoEmojiMaxRows + "]");
        }

        // デコ絵文字を表示するエリアの縦幅を算出する
        mDecoEmojiLinearHeight = mDisplayHeight;
        // デコ／キャリア絵文字タブの縦幅を減算する
        mDecoEmojiLinearHeight -= res.getDimensionPixelSize(R.dimen.tabs_height);

        // 連続モードの場合のみプレビューテキストエリアを減算する
        if (mCurrentInputType != INPUT_TYPE_SINGLE) {
            // プレビューテキストの縦幅を減算する
            mDecoEmojiLinearHeight -= (mDisplayOneLineHeight);
        }
        // Spinnerの縦幅を減算する
        mDecoEmojiLinearHeight -= res.getDimensionPixelSize(R.dimen.category_name_height);

        // 連続モードの場合のみボタンエリアを減算する
        if (mCurrentInputType != INPUT_TYPE_SINGLE) {
            // 下部のボタンの縦幅を減算する
            mDecoEmojiLinearHeight -= res.getDimensionPixelSize(R.dimen.buttons_height);
        }
        // デコ絵文字の横幅を算出する
        mDecoEmojiWidth = (int) Math.floor(mDisplayWidth / mDecoEmojiMaxCols);

        // デコ絵文字一覧の右端を詰めさせる為、1行の右端のデコ絵文字の横幅を算出する
        mDecoEmojiWidthEndOfLine = mDecoEmojiWidth + (mDisplayWidth % mDecoEmojiMaxCols);

        // デコ絵文字の縦幅を算出する
        mDecoEmojiHeight = mDecoEmojiLinearHeight / mDecoEmojiMaxRows;

        // ページ間で隙間を埋める為、1画面の最終のデコ絵文字(行)の縦幅を算出する
        mDecoEmojiHeightEndOfLine = mDecoEmojiHeight
                + (mDecoEmojiLinearHeight % mDecoEmojiMaxRows);

        // デコ絵文字一覧の最大描画数を算出する
        mDecoEmojiMaxDrawableCount = mDecoEmojiMaxRows * mDecoEmojiMaxCols;

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "initView() mDecoEmojiMaxDrawableCount["
                    + mDecoEmojiMaxDrawableCount + "]");
        }

        // 土台の縦幅を設定する
        mViewGroup = (ViewGroup) inflater.inflate(R.layout.decoemojilist, null);
        mListViewBase = (ViewGroup) mViewGroup.findViewById(R.id.base_linear);
        mListViewBase.setMinimumWidth(mDisplayWidth);
        mListViewBase.setMinimumHeight(mDisplayHeight);

        // プレビューテキストを設定する
        mPreviewText = (TextView) mViewGroup.findViewById(R.id.preview_text);
        mPreviewText.setMinimumWidth(mDisplayWidth);
        mPreviewText.setMinimumHeight(mDisplayOneLineHeight);
        mPreviewText.setGravity(Gravity.CENTER_VERTICAL);

        // カテゴリ名称を設定する
        mCategoryName = (Spinner) mViewGroup.findViewById(R.id.category_name_spinner);
        mCategoryName.setMinimumWidth(mDisplayWidth);

        // デコ絵文字一覧を設定する
        mBaseScroll = (ScrollView) mViewGroup.findViewById(R.id.base_scroll);
        ViewGroup.LayoutParams params = new LinearLayout.LayoutParams(
                ViewGroup.LayoutParams.FILL_PARENT, mDecoEmojiLinearHeight);
        mBaseScroll.setLayoutParams(params);
        LinearLayout linearPrevious = (LinearLayout) mViewGroup
                .findViewById(R.id.list_previous_linear);
        linearPrevious.setMinimumHeight(mDecoEmojiLinearHeight);
        LinearLayout linearNow = (LinearLayout) mViewGroup.findViewById(R.id.list_now_linear);
        linearNow.setMinimumHeight(mDecoEmojiLinearHeight);
        LinearLayout linearNext = (LinearLayout) mViewGroup
                .findViewById(R.id.list_next_linear);
        linearNext.setMinimumHeight(mDecoEmojiLinearHeight);

        // 各種ボタンを設定する
        mEnterButton = (ImageView) mViewGroup.findViewById(R.id.enter_button);
        mReadMoreButtonBack = (ImageView) mViewGroup.findViewById(R.id.read_more_back_button);
        mReadMoreButtonNext = (ImageView) mViewGroup.findViewById(R.id.read_more_next_button);
        mBackSpaceButton = (ImageView) mViewGroup.findViewById(R.id.back_space_button);

        setButtonsParam(mEnterButton, R.drawable.button_ok);
        setButtonsParam(mReadMoreButtonBack, R.drawable.button_allow_up);
        setButtonsParam(mReadMoreButtonNext, R.drawable.button_allow_down);
        setButtonsParam(mBackSpaceButton, R.drawable.button_del);

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "initView() end.");
        }

        return mViewGroup;
    }

    /**
     * 各種ボタンの共通項目を設定する.
     *
     * @param targetImageView
     *            ターゲットとなるImageView
     * @param resourceId
     *            ターゲットとなるViewのリソースID
     */
    private void setButtonsParam(ImageView targetImageView, int resourceId) {
        // 前／次ページボタンの横幅を算出する
        int buttonsWidth = (int) Math.floor(mDisplayWidth / Const.MAX_BUTTONS_COLS);

        // ターゲットに対してパラメータを設定する
        targetImageView.setBackgroundResource(R.drawable.buttons_background_type2);
        targetImageView.setImageResource(resourceId);
        targetImageView.setMinimumHeight(mDisplayOneLineHeight);
        targetImageView.setMinimumWidth(buttonsWidth);
        targetImageView.setMaxWidth(buttonsWidth);
        targetImageView.setScaleType(ImageView.ScaleType.CENTER);
    }

    /**
     * mDisplayWidthの取得.
     *
     * @return mDisplayWidth
     */
    public int getDisplayWidth() {
        return mDisplayWidth;
    }

    /**
     * mDecoEmojiLinearHeightの取得
     *
     * @return mDecoEmojiLinearHeight
     */
    public int getDecoEmojiLinearHeight() {
        return mDecoEmojiLinearHeight;
    }

    /**
     * mDecoEmojiMaxDrawableCountの取得
     *
     * @return mDecoEmojiMaxDrawableCount
     */
    public int getDecoEmojiMaxDrawableCount() {
        return mDecoEmojiMaxDrawableCount;
    }

    /**
     * mDecoEmojiWidthの取得
     *
     * @return mDecoEmojiWidth
     */
    public int getDecoEmojiWidth() {
        return mDecoEmojiWidth;
    }

    /**
     * mDecoEmojiWidthEndOfLineの取得
     *
     * @return mDecoEmojiWidthEndOfLine
     */
    public int getDecoEmojiWidthEndOfLine() {
        return mDecoEmojiWidthEndOfLine;
    }

    /**
     * mDecoEmojiHeightの取得
     *
     * @return mDecoEmojiHeight
     */
    public int getDecoEmojiHeight() {
        return mDecoEmojiHeight;
    }

    /**
     * mDecoEmojiHeightEndOfLineの取得
     *
     * @return mDecoEmojiHeightEndOfLine
     */
    public int getDecoEmojiHeightEndOfLine() {
        return mDecoEmojiHeightEndOfLine;
    }

    /**
     * mDecoEmojiMaxColsの取得
     *
     * @return mDecoEmojiMaxCols
     */
    public int getDecoEmojiMaxCols() {
        return mDecoEmojiMaxCols;
    }

    /**
     * mDecoEmojiMaxRowsの取得
     *
     * @return mDecoEmojiMaxRows
     */
    public int getDecoEmojiMaxRows() {
        return mDecoEmojiMaxRows;
    }
}
